CONTENTS OF THIS FILE
---------------------

* Introduction
* Requirements
* Installation
* Configuration
* Features


INTRODUCTION
------------

This module provides a block to display a calendar powered by FullCalendar 5.

It creates a block called 'FullCalendar block' which accepts event source as a json (URL) feed.

The URL can be a relative or absolute link. You may use a relative link if the event source is

* For a full description of the module visit:
  https://www.drupal.org/project/fullcalendar_block

* To submit bug reports and feature suggestions, or to track changes visit:
  https://www.drupal.org/project/issues/fullcalendar_block

REQUIREMENTS
------------

This module requires FullCalendar library (https://github.com/fullcalendar/fullcalendar).

It may also make use of DOMPurify (https://github.com/cure53/DOMPurify) to handle sanitization of HTML when rendering
custom event descriptions in a dialog popup to prevent potential XSS attacks.

As well as support for the Moment (https://fullcalendar.io/docs/moment-plugin) and RRule (https://fullcalendar.io/docs/rrule-plugin) plugins.

INSTALLATION
------------

* Install via Composer.
  You can install this module as a normal installation process of Composer.

  Optionally, if you'd like to automatically install the external libraries into your local Drupal instance, then you may add `drupal/fullcalendar_block` as a `drupal-libraries-dependencies` of the [zodiacmedia/drupal-libraries-installer](https://github.com/zodiacmedia/drupal-libraries-installer#installing-libraries-declared-by-other-packages) plugin.
  * Ensure that packages of type drupal-library are [properly configured to install](https://www.drupal.org/docs/develop/using-composer/using-composer-to-install-drupal-and-manage-dependencies#s-define-the-directories-to-which-drupal-projects-should-be-downloaded) to the `/libraries` directory.
  * Add the composer dependency: `composer require zodiacmedia/drupal-libraries-installer`.
  * Add the library as a dependency:
    ```json
    {
      "extra": {
        "drupal-libraries-dependencies": [
          "drupal/fullcalendar_block"
        ]
      }
    }
    ```

Then run `composer install` as normal.

* Install via zip file.

### MANUAL INSTALLATION

If you'd like to manually install the external Javascript libraries into your local Drupal instance, you may download and extract the following into your Drupal root directory:

| URL                                                                 | Destination                      |
|---------------------------------------------------------------------|----------------------------------|
| https://registry.npmjs.org/fullcalendar/-/fullcalendar-5.11.1.tgz   | `/libraries/drupal-fullcalendar` |
| https://registry.npmjs.org/dompurify/-/dompurify-2.3.8.tgz          | `/libraries/DOMPurify`           |
| https://registry.npmjs.org/moment/-/moment-2.26.0.tgz               | `/libraries/moment`              |
| https://registry.npmjs.org/rrule/-/rrule-2.6.4.tgz                  | `/libraries/rrule`               |
| https://registry.npmjs.org/@fullcalendar/rrule/-/rrule-5.11.0.tgz   | `/libraries/fullcalendar-moment` |
| https://registry.npmjs.org/@fullcalendar/moment/-/moment-5.11.0.tgz | `/libraries/fullcalendar-rrule`  |

If the required external libraries are unavailable locally, then they will be loaded from a CDN (usually https://unpkg.com).

CONFIGURATION
-------------

1. Go to the Block layout configuration page (`/admin/structure/block`).
2. Click the 'Place block' button in the area you want the calendar block take place.
3. In the pop-up dialog search 'FullCalendar block' and then place the block.
4. Click the 'Configure' button of the block to set up the block.
5. The 'Event source URL' is mandatory which is the JSON feed URL providing the event data of the calendar.
6. The 'Advanced settings' provides the capability to add or modify any options of the FullCalendar, which has to be valid JSON/YAML format text. For example, following will set the initial date at 2022-05-01,

   ```yaml
   initialDate: '2022-05-01'
   ```

8. The 'Advanced Drupal settings' provides support for more advanced Fullcalendar behaviour relating to the dialog, which has to be valid JSON/YAML format text. Available options are as follows:
   ```yaml
   # The dialog modal type.
   dialog_type: 'modal'
   # Additional options to pass through to the Drupal modal.
   dialog_options:
     # Disable Drupal's default autoResize feature on all blocks.
     autoResize: false
   # Whether to enable integration with jquery_ui_draggable if installed.
   draggable: false
   # Options to pass over to jquery_ui_draggable.
   draggable_options: {}
   # Whether to enable integration with jquery_ui_resizable if installed.
   resizable: false
   # Options to pass over to jquery_ui_resizable.
   resizable_options: {}
   # Show event description in dialog window.
   # If enabled, the description will show in a dialog window upon clicking the event.
   description_popup: false
   # Event description field name to use for the popup dialog.
   description_field: 'des'
   ```

8. After saving the configurations, you might need to clear the cache to apply the changes, but it's typically not
   necessary.

### Development
1. During development, you may switch to non-minified assets by specifying the following in your local `settings.php` and clearing the Drupal cache:

```php
// Enable non-minified assets.
$settings['fullcalendar_block.dev_mode'] = TRUE;
```

2. There is a hook called 'hook_fullcalendar_block_settings_alter' for other modules to modify the block settings, includeing the calendar options.
More details about this hook see 
https://git.drupalcode.org/project/fullcalendar_block/-/blob/1.0.x/fullcalendar_block.api.php

3. JavaScript events.
 - fullcalendar_block.beforebuild
   An event to let other modules know that a calendar will be built.
    Parameters:
      - event
        The event object.
      - calendarOptions
        The calendar opetions object.

 - fullcalendar_block.build
   An event to let other modules know when a calendar has been built.
   Parameters:
    - event
        The event object.
    - blockInstance
        The block instance object.
 
FEATURES:
-------------
* FullCalendar 5.
* Fully configurable via the block setting.
* Support simple recurring events out of box.
* Support loading events via Ajax once needed.
* Support the moment and RRule fullcalendar plugins

